import 'dart:math';

import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'carousel_arrow_btn.dart';

class ImageViewer extends StatefulWidget {
  final List<String> imagePaths;
  final int currentImageIndex;

  const ImageViewer({
    Key? key,
    required this.imagePaths,
    required this.currentImageIndex,
  }) : super(key: key);

  @override
  State<ImageViewer> createState() => _ImageViewerState();
}

class _ImageViewerState extends State<ImageViewer> {
  late final PageController _pageController;
  int _currentPageIndex = 0;

  @override
  void initState() {
    _currentPageIndex = widget.currentImageIndex;
    _pageController = PageController(
      initialPage: widget.currentImageIndex,
    );
    super.initState();
  }

  @override
  void dispose() {
    _pageController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.black,
      body: SafeArea(
        child: Stack(
          children: [
            ExtendedImageGesturePageView.builder(
              onPageChanged: (index) {
                setState(() {
                  _currentPageIndex = index;
                });
              },
              controller: _pageController,
              itemBuilder: (BuildContext context, int index) {
                return ExtendedImageSlidePage(
                  resetPageDuration: const Duration(
                    milliseconds: 100,
                  ),
                  slideEndHandler: (
                    Offset offset, {
                    ExtendedImageSlidePageState? state,
                    ScaleEndDetails? details,
                  }) {
                    if (state == null) {
                      return false;
                    }

                    return offset.distance >
                        Offset(state.pageSize.width, state.pageSize.height)
                                .distance /
                            20;
                  },
                  slidePageBackgroundHandler: (Offset offset, Size pageSize) {
                    const Color color = Colors.black;
                    final double opacity = offset.distance /
                        (Offset(pageSize.width, pageSize.height).distance /
                            2.0);
                    return color.withOpacity(min(1.0, max(opacity, 0.0)));
                  },
                  child: ExtendedImage.network(
                    widget.imagePaths[index],
                    height: double.infinity,
                    fit: BoxFit.contain,
                    enableSlideOutPage: true,
                    mode: ExtendedImageMode.gesture,
                    initGestureConfigHandler: (state) {
                      return GestureConfig(
                        minScale: 1,
                        animationMinScale: 0.7,
                        maxScale: 3.0,
                        animationMaxScale: 3.5,
                        speed: 1.0,
                        inertialSpeed: 100.0,
                        initialScale: 1.0,
                        inPageView: false,
                        initialAlignment: InitialAlignment.center,
                      );
                    },
                  ),
                );
              },
              itemCount: widget.imagePaths.length,
              scrollDirection: Axis.horizontal,
            ),
            if (_currentPageIndex > 0)
              Align(
                alignment: Alignment.centerLeft,
                child: Container(
                  padding: const EdgeInsets.all(5),
                  child: Transform.rotate(
                    angle: pi,
                    child: CarouselArrowBtn(
                      onTap: () {
                        _pageController.previousPage(
                          duration: const Duration(milliseconds: 300),
                          curve: Curves.ease,
                        );
                      },
                    ),
                  ),
                ),
              ),
            if (_currentPageIndex < widget.imagePaths.length - 1)
              Align(
                alignment: Alignment.centerRight,
                child: Container(
                  padding: const EdgeInsets.all(5),
                  child: CarouselArrowBtn(
                    onTap: () {
                      _pageController.nextPage(
                        duration: const Duration(milliseconds: 300),
                        curve: Curves.ease,
                      );
                    },
                  ),
                ),
              ),
            Positioned(
              top: 5,
              left: 10,
              child: IconButton(
                onPressed: () => Navigator.pop(context),
                icon: const Icon(
                  Icons.cancel_rounded,
                  size: 30,
                  color: Colors.white,
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}
